C ********************************************************************
C * EDiT - Electronic Dipole-induced Transitions                     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE BDIAG                                                *
C *                                                                  *
C *  Routine of EDiT/CEDiT                                           *
C *  version for IBM-PC MS-DOS, Microsoft FORTRAN and                *
C *              IBM-RISC UNIX, AIX XL FORTRAN                       * 
C *                                                                  *
C *  Usage......: Calculates the blockdiagonal elements of the       *
C *               transition matrix.                                 *
C *                                                                  *
C *  Authors....: Ruedi Rytz                                         *
C *                                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: June 1992                                          *
C *                                                                  *
C *  Revision...: -                                                  *
C *                                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: MATRIX                                             *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE  BDIAG(M, ANZATO, FAO, LAO, POL, NS, NP, 
     .                  POINTER, EXPS, EXPP, EXPD1, EXPD2,
     .                  C1, C2)

      IMPLICIT NONE

      INCLUDE 'sizes'

        DOUBLE PRECISION M(MAXAO,MAXAO)
        INTEGER          ANZATO
        INTEGER          FAO(MAXATO), LAO(MAXATO)
        CHARACTER*1      POL
        INTEGER          NS(MAXATO), NP(MAXATO)
        INTEGER          POINTER(MAXATO)
        DOUBLE PRECISION EXPS(MAXATO), EXPP(MAXATO), EXPD1(MAXATO)
        DOUBLE PRECISION EXPD2(MAXATO), C1(MAXATO), C2(MAXATO)

C CONST
      DOUBLE PRECISION SQ1D3, SQ4D3
      PARAMETER (SQ1D3 = 5.773503D-1, SQ4D3 = 1.154701D0)

C VAR
      INTEGER           I, J, N
      DOUBLE PRECISION  SPOCE, PDOCE, DX2_Y2

C BEGIN
C s nach p Uebergaenge

      DO 50 N = 1, ANZATO

C IF (LAO(N) - FAO(N) + 1 .GE. 4 ) THEN ... bedeutet dass die Uebergaenge
C an diesem Atom nur betrachtet werden, wenn es dort auch p-Orbitale hat.

        IF ( LAO(N) - FAO(N) + 1 .GE. 4 ) THEN

C Es wird nur die obere rechte Haelfte der Matrix berechnet.

          DO 60 I = FAO(N), FAO(N) + 2
            DO 70 J = I + 1, FAO(N) + 3

C IF (I .EQ. FAO(N) THEN.. ist TRUE wenn das AO ein s-Orbital ist.

              IF (I .EQ. FAO(N)) THEN

C J - FAO(N) .. ist 1 fuer ein px-Orbital, 2 fuer ein py-Orbital usw.
C ICHAR('X') ist 88; daraus folgt, dass ICHAR('X') - 87 1 ergibt.
C Es handelt sich also um einen s nach px Uebergang, welcher in X-
C Richtung erlaubt ist. Matrixelemente <s|x|py> sind verboten.

                IF (J - FAO(N) .EQ. ICHAR(POL) - 87) THEN
                  M(I,J) = M(I,J) * SPOCE (NS(POINTER(N)), 
     .                              EXPS(POINTER(N)), EXPP(POINTER(N)))

                  M(J,I) = M(J,I) * SPOCE (NS(POINTER(N)), 
     .                              EXPS(POINTER(N)), EXPP(POINTER(N)))
                ELSE
                  M(I,J) = 0.0D0
                  M(J,I) = 0.0D0
                ENDIF
              ELSE
                M(I,J) = 0.0D0
                M(J,I) = 0.0D0
              ENDIF
70          CONTINUE
60        CONTINUE
        ENDIF
50    CONTINUE

C Ende s nach p Uebergaenge

C p nach d Uebergaenge

      DO 100 N = 1, ANZATO

C IF (LAO(N) - FAO(N) + 1 .EQ. 9 ) THEN ... bedeutet dass die Uebergaenge
C an diesem Atom nur betrachtet werden, wenn es dort auch d-Orbitale hat.

        IF ( LAO(N) - FAO(N) + 1 .EQ. 9 ) THEN

C DO 110 I = FAO(N), FAO(N) + 3... s bis pz - Orbitale werden betrachtet.
C DO 120 J = FAO(N) + 4, FAO(N) + 8... spricht nur die d-Orbitale an.

          DO 110 I = FAO(N), FAO(N) + 3
            DO 120 J = FAO(N) + 4, FAO(N) + 8

C IF (I .EQ. FAO(N)) THEN... ist TRUE, wenn I ein s-Orbital
C s nach d Uebergaenge sind null.

              IF (I .EQ. FAO(N)) THEN
                M(I,J) = 0.0D0
                M(J,I) = 0.0D0
              ENDIF

C IF (I .EQ. FAO(N) + 1) THEN... bedeutet, dass man ein px-Orbital betrachtet.

              IF (I .EQ. FAO(N) + 1) THEN
                DX2_Y2 = C1(POINTER(N)) * 
     .                   PDOCE (NP(POINTER(N)), EXPP(POINTER(N)),
     .                          EXPD1(POINTER(N))) +
     .                   C2(POINTER(N)) * 
     .                   PDOCE (NP(POINTER(N)), EXPP(POINTER(N)),
     .                          EXPD2(POINTER(N)))

                IF (POL .EQ. 'X') THEN

C IF (J .EQ. LAO(N) - 4) THEN... es handelt sich um ein dx^2-y^2-Orbital

                  IF (J .EQ. LAO(N) - 4) THEN
                    M(I,J) = M(I,J) * DX2_Y2
                    M(J,I) = M(J,I) * DX2_Y2
C ELSEIF (J .EQ. LAO(N) - 3) THEN... es handelt sich um ein dz^2-Orbital

                  ELSEIF (J .EQ. LAO(N) - 3) THEN
                    M(I,J) = - M(I,J) * DX2_Y2 * SQ1D3
                    M(J,I) = - M(J,I) * DX2_Y2 * SQ1D3
                  ELSE
                    M(I,J) = 0.0D0
                    M(J,I) = 0.0D0
                  ENDIF

                ELSEIF (POL .EQ. 'Y') THEN
                  IF (J .EQ. LAO(N) - 2) THEN
                    M(I,J) = M(I,J) * DX2_Y2
                    M(J,I) = M(J,I) * DX2_Y2
                  ELSE
                    M(I,J) = 0.0D0
                    M(J,I) = 0.0D0
                  ENDIF
                ELSE
                  IF (J .EQ. LAO(N) - 1) THEN
                    M(I,J) = M(I,J) * DX2_Y2
                    M(J,I) = M(J,I) * DX2_Y2
                  ELSE
                    M(I,J) = 0.0D0
                    M(J,I) = 0.0D0
                  ENDIF
                ENDIF
              ENDIF

C IF (I .EQ. FAO(N) + 2) THEN... es handelt sich um ein py-Orbital

              IF (I .EQ. FAO(N) + 2) THEN

                IF (POL .EQ. 'X') THEN
                  IF (J .EQ. LAO(N) - 2) THEN
                    M(I,J) = M(I,J) * DX2_Y2
                    M(J,I) = M(J,I) * DX2_Y2
                  ELSE
                    M(I,J) = 0.0D0
                    M(J,I) = 0.0D0
                  ENDIF

                ELSEIF (POL .EQ. 'Y') THEN
                  IF (J .EQ. LAO(N) - 4) THEN
                    M(I,J) = - M(I,J) * DX2_Y2
                    M(J,I) = - M(J,I) * DX2_Y2
                  ELSE IF (J .EQ. LAO(N) - 3) THEN
                    M(I,J) = - M(I,J) * DX2_Y2 * SQ1D3
                    M(J,I) = - M(J,I) * DX2_Y2 * SQ1D3
                  ELSE
                    M(I,J) = 0.0D0
                    M(J,I) = 0.0D0
                  ENDIF

                ELSE
                  IF (J .EQ. LAO(N)) THEN
                    M(I,J) = M(I,J) * DX2_Y2
                    M(J,I) = M(J,I) * DX2_Y2
                  ELSE
                    M(I,J) = 0.0D0
                    M(J,I) = 0.0D0
                  ENDIF
                ENDIF

              ENDIF

C IF (I .EQ. FAO(N) + 3) THEN... es handelt sich um ein pz-Orbital

              IF (I .EQ. FAO(N) + 3) THEN

                IF (POL .EQ. 'X') THEN
                  IF (J .EQ. LAO(N) - 1) THEN
                    M(I,J) = M(I,J) * DX2_Y2
                    M(J,I) = M(J,I) * DX2_Y2
                  ELSE
                    M(I,J) = 0.0D0
                    M(J,I) = 0.0D0
                  ENDIF

                ELSEIF (POL .EQ. 'Y') THEN
                  IF (J .EQ. LAO(N)) THEN
                    M(I,J) = M(I,J) * DX2_Y2
                    M(J,I) = M(J,I) * DX2_Y2
                  ELSE
                    M(I,J) = 0.0D0
                    M(J,I) = 0.0D0
                  ENDIF

                ELSE
                  IF (J .EQ. LAO(N) - 3) THEN
                    M(I,J) = M(I,J) * DX2_Y2 * SQ4D3
                    M(J,I) = M(J,I) * DX2_Y2 * SQ4D3
                  ELSE
                    M(I,J) = 0.0D0
                    M(J,I) = 0.0D0
                  ENDIF
                ENDIF
              ENDIF
120         CONTINUE
110       CONTINUE

C Explizites Nullsetzen der d-d Uebergaenge
          DO 130 I = FAO(N)+4, FAO(N) + 7
            DO 140 J = I + 1, FAO(N) + 8
              M(I,J) = 0.0D0
              M(J,I) = 0.0D0
140         CONTINUE
130       CONTINUE
C Ende explizietes Nullsetzen der d-d Uebergaenge

        ENDIF
100   CONTINUE

C Ende p nach d Uebergaenge

      RETURN
      END
C END
