C ********************************************************************
C * EDiT - Electronic Dipole-induced Transitions                     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE DOPELE                                               *
C *                                                                  *
C *  Routine of EDiT/CEDiT                                           *
C *  version for IBM-PC MS-DOS, Microsoft FORTRAN and                *
C *              IBM-RISC UNIX, AIX XL FORTRAN                       * 
C *                                                                  *
C *  Usage......: Calculates <d|O|p> offdiagonal elements of the     *
C *               transition matrix. The operator operates to the    *
C *               left function.                                     *
C *                                                                  *
C *  Authors....: Ruedi Rytz                                         *
C *                                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: August 1992                                        *
C *  Revision...: -                                                  *
C *                                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: OBDIAG                                             *
C *                                                                  *
C *  Calls......: ABFNS                                              *
C *               LOVLAP                                             *
C *               PDOVL                                              *
C *               PPOVL                                              *
C *               FPOVL                                              *
C *                                                                  *
C ********************************************************************

      SUBROUTINE DOPELE(M, HQZ, N2, SK11, SK12, SK2, C1, C2,
     .                  R, PTR, DTR, FTR, LDIST, POL, CURRI,
     .                  CURRJ)

      IMPLICIT NONE

        DOUBLE PRECISION  M
        INTEGER           HQZ, N2
        DOUBLE PRECISION  SK11, SK12, SK2, C1, C2, R, PTR(9), DTR(25)
        DOUBLE PRECISION  FTR(49), LDIST
        CHARACTER*1       POL
        INTEGER           CURRI, CURRJ

C CONST
      INCLUDE 'dconst.inc'

C VAR
      DOUBLE PRECISION  A(20), B(20), A1(20), B1(20)
      DOUBLE PRECISION  DPOS1, DPOS2, DPOP1, DPOP2, DPO1(3,5), DPO2(3,5)
      DOUBLE PRECISION  PPOS1, PPOS2, PPOP1, PPOP2, PPO1(3,3), PPO2(3,3)
      DOUBLE PRECISION  FPOS1, FPOS2, FPOP1, FPOP2, FPO1(3,7), FPO2(3,7)
      INTEGER           L1, L2, N1, MAX, ML
      DOUBLE PRECISION  NORM1, NORM2

      INCLUDE 'dfns.inc'
      INCLUDE 'dnorm.inc'

C BEGIN

C Berechne zuerst den "normalen" d-p-Overlap
      N1 = HQZ
      L1 = 2
      L2 = 1
      ML = 0
      MAX = N1 + N2

      CALL ABFNS(A, B, SK11, SK2, R, MAX)
      CALL LOVLAP(DPOS1, A, B, SK11, SK2, R, L1, L2,
     .            ML, N1, N2)
      CALL ABFNS(A1, B1, SK12, SK2, R, MAX)
      CALL LOVLAP(DPOS2, A1, B1, SK12, SK2, R, L1, L2,
     .            ML, N1, N2)

      ML = 1

      CALL LOVLAP(DPOP1, A, B, SK11, SK2, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(DPOP2, A1, B1, SK12, SK2, R, L1, L2,
     .            ML, N1, N2)

      DPOP1 = -DPOP1
      DPOP2 = -DPOP2

      CALL PDOVL(DPO1, DPOS1, DPOP1, PTR, DTR)
      CALL PDOVL(DPO2, DPOS2, DPOP2, PTR, DTR)

C Berechne den p-p-Overlap nachdem man den Operator auf das
C d-Orbital angewandt hat.

      N1 = N1 + 1
      L1 = 1
      ML = 0
      MAX = N1 + N2

      CALL ABFNS(A, B, SK11, SK2, R, MAX)
      CALL LOVLAP(PPOS1, A, B, SK11, SK2, R, L1, L2,
     .            ML, N1, N2)
      CALL ABFNS(A1, B1, SK12, SK2, R, MAX)
      CALL LOVLAP(PPOS2, A1, B1, SK12, SK2, R, L1, L2,
     .            ML, N1, N2)

      PPOS1 = -PPOS1
      PPOS2 = -PPOS2

      ML = 1
      CALL LOVLAP(PPOP1, A, B, SK11, SK2, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(PPOP2, A1, B1, SK12, SK2, R, L1, L2,
     .            ML, N1, N2)

      CALL PPOVL(PPO1, PPOS1, PPOP1, PTR)
      CALL PPOVL(PPO2, PPOS2, PPOP2, PTR)

C Berechne den f-p-Overlap nachdem man den Operator auf das d-Orbital
C angewandt hat.

      L1 = 3
      ML = 0

      CALL LOVLAP(FPOS1, A, B, SK11, SK2, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(FPOS2, A1, B1, SK12, SK2, R, L1, L2,
     .            ML, N1, N2)

      FPOS1 = -FPOS1
      FPOS2 = -FPOS2

      ML = 1

      CALL LOVLAP(FPOP1, A, B, SK11, SK2, R, L1, L2,
     .            ML, N1, N2)
      CALL LOVLAP(FPOP2, A1, B1, SK12, SK2, R, L1, L2,
     .            ML, N1, N2)

      CALL FPOVL(FPO1, FPOS1, FPOP1, PTR, FTR)
      CALL FPOVL(FPO2, FPOS2, FPOP2, PTR, FTR)

      IF (CURRI .EQ. 5) THEN

        IF (POL .EQ. 'X') THEN
          M = M * (C1 * I5X(PPO1(CURRJ-1,1), DPO1(CURRJ-1,1),
     .                      FPO1(CURRJ-1,6), FPO1(CURRJ-1,2), SK11) +
     .             C2 * I5X(PPO2(CURRJ-1,1), DPO2(CURRJ-1,1),
     .                      FPO2(CURRJ-1,6), FPO2(CURRJ-1,2), SK12))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * (C1 * I5Y(PPO1(CURRJ-1,2), DPO1(CURRJ-1,1),
     .                      FPO1(CURRJ-1,7), FPO1(CURRJ-1,3), SK11) +
     .             C2 * I5Y(PPO2(CURRJ-1,2), DPO2(CURRJ-1,1),
     .                      FPO2(CURRJ-1,7), FPO2(CURRJ-1,3), SK12))
        ELSE
          M = M * (C1 * XYZ(DPO1(CURRJ-1,1), FPO1(CURRJ-1,5), SK11) +
     .             C2 * XYZ(DPO2(CURRJ-1,1), FPO2(CURRJ-1,5), SK12))
        ENDIF

      ELSEIF (CURRI .EQ. 6) THEN

        IF (POL .EQ. 'X') THEN
          M = M * (C1 * I6XY(PPO1(CURRJ-1,1), DPO1(CURRJ-1,2),
     .                       FPO1(CURRJ-1,2), SK11) +
     .             C2 * I6XY(PPO2(CURRJ-1,1), DPO2(CURRJ-1,2),
     .                       FPO2(CURRJ-1,2), SK12))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * (C1 * I6XY(PPO1(CURRJ-1,2), DPO1(CURRJ-1,2),
     .                       FPO1(CURRJ-1,3), SK11) +
     .             C2 * I6XY(PPO2(CURRJ-1,2), DPO2(CURRJ-1,2),
     .                       FPO2(CURRJ-1,3), SK12))
        ELSE
          M = M * (C1 * I6Z(PPO1(CURRJ-1,3), DPO1(CURRJ-1,2),
     .                      FPO1(CURRJ-1,1), SK11) +
     .             C2 * I6Z(PPO2(CURRJ-1,3), DPO2(CURRJ-1,2),
     .                      FPO2(CURRJ-1,1), SK12))
        ENDIF

      ELSEIF (CURRI .EQ. 7) THEN

        IF (POL .EQ. 'X') THEN
          M = M * (C1 * I7X(PPO1(CURRJ-1,2), DPO1(CURRJ-1,3),
     .                      FPO1(CURRJ-1,7), FPO1(CURRJ-1,3), SK11) +
     .             C2 * I7X(PPO2(CURRJ-1,2), DPO2(CURRJ-1,3),
     .                      FPO2(CURRJ-1,7), FPO2(CURRJ-1,3), SK12))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * (C1 * I7Y(PPO1(CURRJ-1,1), DPO1(CURRJ-1,3),
     .                      FPO1(CURRJ-1,6), FPO1(CURRJ-1,2), SK11) +
     .             C2 * I7Y(PPO2(CURRJ-1,1), DPO2(CURRJ-1,3),
     .                      FPO2(CURRJ-1,6), FPO2(CURRJ-1,2), SK12))
        ELSE
          M = M * (C1 * XYZ(DPO1(CURRJ-1,3), FPO1(CURRJ-1,4), SK11) +
     .             C2 * XYZ(DPO2(CURRJ-1,3), FPO2(CURRJ-1,4), SK12))
        ENDIF

      ELSEIF (CURRI .EQ. 8) THEN

        IF (POL .EQ. 'X') THEN
          M = M * (C1 * I8X(PPO1(CURRJ-1,3), DPO1(CURRJ-1,4),
     .                      FPO1(CURRJ-1,5), FPO1(CURRJ-1,1), SK11) +
     .             C2 * I8X(PPO2(CURRJ-1,3), DPO2(CURRJ-1,4),
     .                      FPO2(CURRJ-1,5), FPO2(CURRJ-1,1), SK12))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * (C1 * XYZ(DPO1(CURRJ-1,4), FPO1(CURRJ-1,4), SK11) +
     .             C2 * XYZ(DPO2(CURRJ-1,4), FPO2(CURRJ-1,4), SK12))
        ELSE
          M = M * (C1 * I8Z(PPO1(CURRJ-1,1), DPO1(CURRJ-1,4),
     .                      FPO1(CURRJ-1,2), SK11) +
     .             C2 * I8Z(PPO2(CURRJ-1,1), DPO2(CURRJ-1,4),
     .                      FPO2(CURRJ-1,2), SK12))
        ENDIF

      ELSEIF (CURRI .EQ. 9) THEN

        IF (POL .EQ. 'X') THEN
          M = M * (C1 * XYZ(DPO1(CURRJ-1,5), FPO1(CURRJ-1,4), SK11) +
     .             C2 * XYZ(DPO2(CURRJ-1,5), FPO2(CURRJ-1,4), SK12))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * (C1 * I9Y(PPO1(CURRJ-1,3), DPO1(CURRJ-1,5),
     .                      FPO1(CURRJ-1,5), FPO1(CURRJ-1,1), SK11) +
     .             C2 * I9Y(PPO2(CURRJ-1,3), DPO2(CURRJ-1,5),
     .                      FPO2(CURRJ-1,5), FPO2(CURRJ-1,1), SK12))
        ELSE
          M = M * (C1 * I9Z(PPO1(CURRJ-1,2), DPO1(CURRJ-1,5),
     .                      FPO1(CURRJ-1,3), SK11) +
     .             C2 * I9Z(PPO2(CURRJ-1,2), DPO2(CURRJ-1,5),
     .                      FPO2(CURRJ-1,3), SK12))
        ENDIF

      ENDIF

      RETURN
      END
C END
