C ********************************************************************
C * EDiT - Electronic Dipole-induced Transitions                     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE POPELE                                               *
C *                                                                  *
C *  Routine of EDiT/CEDiT                                           *
C *  version for IBM-PC MS-DOS, Microsoft FORTRAN and                *
C *              IBM-RISC UNIX, AIX XL FORTRAN                       * 
C *                                                                  *
C *  Usage......: Calculates the <p|O|p> transition elements of      *
C *               transition matrix.                                 *
C *                                                                  *
C *  Authors....: Ruedi Rytz                                         * 
C *                                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: July 1992                                          *
C *  Revision...: -                                                  *
C *                                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: OBDIAG                                             *
C *                                                                  *
C *  Calls......: ABFNS                                              *
C *               LOVLAP                                             *
C *               PPOVL                                              *
C *               PDOVL                                              *
C *               SPOVL                                              *
C *                                                                  *
C ********************************************************************

      SUBROUTINE POPELE (M ,HQZ, N2, SK1, SK2, R, PTR,
     .                   DTR, LDIST, POL, CURRI, CURRJ)

      IMPLICIT NONE

        DOUBLE PRECISION  M
        INTEGER           HQZ, N2 
        DOUBLE PRECISION  SK1, SK2, R, PTR(9), DTR(25), LDIST
        CHARACTER*1       POL
        INTEGER           CURRI, CURRJ

C CONST
      INCLUDE 'pconst.inc'

C VAR
      INTEGER L1, L2, ML, MAX, N1 
      DOUBLE PRECISION  A(20), B(20), PPOS, PPO(3,3), PPOP, DPOP
      DOUBLE PRECISION  DPOS, DPO(3,5), SPO(3), SPOS
      DOUBLE PRECISION  NORM1, NORM2

      INCLUDE 'pfns.inc'
      INCLUDE 'pnorm.inc'

C BEGIN

C Berechne zuerst den "normalen" p-p-Overlap

      N1 = HQZ

      L1 = 1
      L2 = 1
      ML = 0
      MAX = N1 + N2

      CALL ABFNS(A, B, SK1, SK2, R, MAX)

      CALL LOVLAP(PPOS, A, B, SK1, SK2, R, L1, L2,
     .            ML, N1, N2)
      PPOS = -PPOS
      ML = 1

      CALL LOVLAP(PPOP, A, B, SK1, SK2, R, L1, L2,
     .            ML, N1, N2)

      CALL PPOVL(PPO, PPOS, PPOP, PTR)

C Berechne den d-p Overlap

      N1 = N1 + 1
      L1 = 2
      ML = 0
      MAX = N1 + N2

      CALL ABFNS(A, B, SK1, SK2, R, MAX)

      CALL LOVLAP(DPOS, A, B, SK1, SK2, R, L1, L2,
     .            ML, N1, N2)

      ML = 1

      CALL LOVLAP(DPOP, A, B, SK1, SK2, R, L1, L2,
     .            ML, N1, N2)

      DPOP = -DPOP

      CALL PDOVL(DPO, DPOS, DPOP, PTR, DTR)
      
C Berechne den s-p Overlap

      L1 = 0
      ML = 0

      CALL LOVLAP(SPOS, A, B, SK1, SK2, R, L1, L2,
     .            ML, N1, N2)

      CALL SPOVL(SPO, SPOS, PTR)

      IF (CURRI .EQ. 2) THEN

        IF (POL .EQ. 'X') THEN
          M = M * I2XX(DPO(CURRJ-1,1), DPO(CURRJ-1,2), 
     .                 SPO(CURRJ-1), PPO(1,CURRJ-1))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * XY(DPO(CURRJ-1,3), PPO(1,CURRJ-1))
        ELSE
          M = M * XY(DPO(CURRJ-1,4), PPO(1,CURRJ-1))
        ENDIF

      ELSEIF (CURRI .EQ. 3) THEN

        IF (POL .EQ. 'X') THEN
          M = M * XY(DPO(CURRJ-1,3), PPO(2,CURRJ-1))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * I3XX(DPO(CURRJ-1,1), DPO(CURRJ-1,2),
     .                 SPO(CURRJ-1), PPO(2,CURRJ-1))
        ELSE
          M = M * XY(DPO(CURRJ-1,5), PPO(2,CURRJ-1))
        ENDIF

      ELSEIF (CURRI .EQ. 4) THEN

        IF (POL .EQ. 'X') THEN
          M = M * XY(DPO(CURRJ-1,4), PPO(3,CURRJ-1))
        ELSEIF (POL .EQ. 'Y') THEN
          M = M * XY(DPO(CURRJ-1,5), PPO(3,CURRJ-1))
        ELSE
          M = M * I4XX(DPO(CURRJ-1,2), SPO(CURRJ-1),
     .                 PPO(3,CURRJ-1))
        ENDIF

      ENDIF

      RETURN
      END
C END
