C ********************************************************************
C * EDiT - Electronic Dipole-induced Transitions                     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE POSELE                                               *
C *                                                                  *
C *  Routine of EDiT/CEDiT                                           *
C *  version for IBM-PC MS-DOS, Microsoft FORTRAN and                *
C *              IBM-RISC UNIX, AIX XL FORTRAN                       *
C *                                                                  *
C *  Usage......: Calculates the <p|O|s> transition elements of the  *
C *               transition matrix.                                 *
C *                                                                  *
C *  Authors....: Ruedi Rytz                                         *
C *                                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: June 1992                                          *
C *  Revision...: -                                                  *
C *                                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: OBDIAG                                             *
C *                                                                  *
C *  Calls......: ABFNS                                              *
C *               LOVLAP                                             *
C *               SPOVL                                              *
C *               SDOVL                                              *
C *                                                                  *
C ********************************************************************

      SUBROUTINE  POSELE(M, HQZ, N2, SK1, SK2,
     .                   R, PTR, DTR, LDIST, POL, CURRI)

      IMPLICIT NONE

        DOUBLE PRECISION M
        INTEGER          HQZ, N2
        DOUBLE PRECISION SK1, SK2, R, PTR(9), DTR(25), LDIST
        CHARACTER*1      POL
        INTEGER          CURRI

C CONST
      INCLUDE 'pconst.inc'

C VAR
      INTEGER          L1, L2, ML, MAX, N1
      DOUBLE PRECISION A(20), B(20), SSOS, DSOS, PSOS, DSO(5), PSO(3)
      DOUBLE PRECISION NORM1, NORM2

      INCLUDE 'pfns.inc'
      INCLUDE 'pnorm.inc'

C BEGIN

C Berechne zuerst den "normalen" p-s-Overlap

      N1 = HQZ

      L1 = 1
      L2 = 0
      ML = 0
      MAX = N1 + N2

      CALL ABFNS(A, B, SK1, SK2, R, MAX)
      CALL LOVLAP(PSOS, A, B, SK1, SK2, R, L1, L2,
     .            ML, N1, N2)

      PSOS = -PSOS
      CALL SPOVL(PSO, PSOS, PTR)

C Berechne den s-s-Overlap nachdem man den Operator auf das p-Orbital
C angewandt hat.

      N1 = N1 + 1
      L1 = 0
      MAX = N1 + N2

      CALL ABFNS(A, B, SK1, SK2, R, MAX)
      CALL LOVLAP(SSOS, A, B, SK1, SK2, R, L1, L2,
     .              ML, N1, N2)

C d nach s Overlap, wenn Operator auf p-Orbital angewandt.

      L1 = 2

      CALL LOVLAP(DSOS, A, B, SK1, SK2, R, L1, L2,
     .            ML, N1, N2)

      CALL SDOVL(DSO, DSOS, DTR)

      IF (CURRI.EQ. 2) THEN

        IF (POL .EQ. 'X') THEN
          M = M * I2XX(DSO(1), DSO(2), SSOS, PSO(1))
        ELSE IF (POL .EQ. 'Y') THEN
          M = M * XY(DSO(3), PSO(1))
        ELSE
          M = M * XY(DSO(4), PSO(1))
        ENDIF

      ELSE IF (CURRI.EQ. 3) THEN

        IF (POL .EQ. 'X') THEN
          M = M * XY(DSO(3), PSO(2))
        ELSE IF (POL .EQ. 'Y') THEN
          M = M * I3XX(DSO(1), DSO(2), SSOS, PSO(2))
        ELSE
          M = M * XY(DSO(5), PSO(2))
        ENDIF

      ELSE IF (CURRI .EQ. 4) THEN

        IF (POL .EQ. 'X') THEN
          M = M * XY(DSO(4), PSO(3))
        ELSE IF (POL .EQ. 'Y') THEN
          M = M * XY(DSO(5), PSO(3))
        ELSE
          M = M * I4XX(DSO(2), SSOS, PSO(3))
        ENDIF

      ENDIF

      RETURN
      END
C END
