C ********************************************************************
C * ICONC - Extended Hueckel Calculations                            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE HUCKEL                                               *
C *                                                                  *
C *  Routine of ICONC                                                *
C *                                                                  *
C *  Usage......: determines occupation numbers, sets up huckel mat- *
C *               rix and controls flow of icon after overlaps have  *
C *               been calculated.                                   *
C *                                                                  *
C *  Authors....: -                                                  *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: -                                                  *
C *  Revision...: last revision 24.08.97/mbs (Martin Braendle)       *
C *  Remarks....: Variables and COMMON block added for FMO calcula-  *
C *               tion. Calling of the FMO-subroutines as in BIGICON *
C *               added. If one does FMO calculation, no charge      *
C *               iteration is possible, because the charges distri- *
C *               buted on the molecule after charge iteration are   *
C *               not consistent with the charges on the fragments.  *
C *               So, after FMO calculation HUCKEL returns immedia-  *
C *               ly to MATRIX.                                      *
C *               For calculating the energy matrices, storing of    *
C *               the H(I,I)s is now done in the arrays HSS, HPP and *
C *               HDD of the COMMON block ENERGY. The previous solu- *
C *               tion of assigning the H(I,I) to the cartesian co-  *
C *               ordinate arrays is not good, because they are used *
C *               when the subroutine INPRT prints the fragment co-  *
C *               ordinates. 17.11.90/mbs                            *
C *               Calling of Gram-Schmidt transformation of the      *
C *               overlap matrix exported to subroutine HUCK because *
C *               the raw overlap integrals are needed by the sub-   *
C *               routines FMOS and FMOT. 19.11.90/mbs               *
C *                                                                  *
C *  Called from: MATRIX                                             *
C *                                                                  *
C *  Calls......: PEGLEG                                             *
C *               NEWDIA                                             *
C *               FMOS                                               *
C *               HUCK                                               *
C *               FMOR                                               *
C *               FMOSRT                                             *
C *               OUTPUT                                             *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE HUCKEL(H,S,MAD,C,SP,             MAXD,COUL0,W,IOCC,HDG,
     . H1,H2,H3,H4,H5,H6,H7,H8,H9, S1,S2,S3,S4,S5,S6,S7,S8,S9,
     . NDIM,    NC,NATOM,NTYPE,NHDG,ND1,ND2,ND3,ND4,ND5,ND6,ND7,ND8,ND9)
 
      INCLUDE 'iconpara.for'
 
      IMPLICIT REAL*8(A-H,O-Z)
 
      DIMENSION H(NDIM,NDIM),S(NDIM,NDIM),MAD(NTYPE,NTYPE),C(NC),
     .          SP(NDIM),MAXD(NDIM),COUL0(NDIM),W(NDIM),IOCC(NDIM),
     .          HDG(NHDG)
      REAL*8 MAD
      REAL*4 IOCC
 
      INTEGER ND1,ND2,ND3,ND4,ND5,ND6,ND7,ND8,ND9
      DIMENSION H1(ND1,ND1),S1(ND1,ND1)
      DIMENSION H2(ND2,ND2),S2(ND2,ND2)
      DIMENSION H3(ND3,ND3),S3(ND3,ND3)
      DIMENSION H4(ND4,ND4),S4(ND4,ND4)
      DIMENSION H5(ND5,ND5),S5(ND5,ND5)
      DIMENSION H6(ND6,ND6),S6(ND6,ND6)
      DIMENSION H7(ND7,ND7),S7(ND7,ND7)
      DIMENSION H8(ND8,ND8),S8(ND8,ND8)
      DIMENSION H9(ND9,ND9),S9(ND9,ND9)
 
      COMMON/TITLE/AB(10)
 
      COMMON /ATOM/EXPS(MXATOM),EXPP(MXATOM),EXPD(MXATOM),EXPD2(MXATOM),
     .             C1(MXATOM),C2(MXATOM),COULS(MXATOM),COULP(MXATOM),
     .             COULD(MXATOM),X(MXATOM),Y(MXATOM),Z(MXATOM),
     .             KEY(MXATOM),AC(MXATOM),VELEC(MXATOM),NS(MXATOM),
     .             NP(MXATOM),ND(MXATOM)
      INTEGER AC,VELEC,KEY
 
      COMMON/CNTRL/CON,PEEP,COULH,NH,NA,NATM,KA,NELEC,METH,IPRINT,
     .             IPUNCH,L1,L2,L3,L4,L5,L6,L7,L8,L9,ONEMAT,ITERAT
      LOGICAL*1 L1,L2,L3,L4,L5,L6,L7,L8,L9,ONEMAT,ITERAT
 
      COMMON/OUT/PRT(24),PUN(24),IOVPOP(24),IENRGY(24)
      LOGICAL*1 PRT,PUN
      INTEGER IOVPOP,IENRGY
 
      COMMON/FMO/FMO(MXATOM),FMOCH(9),SORA(MXATOM),SORB(MXATOM),
     .           NFRAG(9),NUMBAT(9),IFMOPT(4*MXATOM),IBCKPT(MXATOM),
     .           PRIFMO(7)
      INTEGER FMO,FMOCH,SORA,SORB
      LOGICAL*1 PRIFMO
 
      COMMON/ITPARM/DAMP1,DAMP2,DAMP3,LAMPRI,DELTAC,SENSE,MAXCYC,
     .              PRTCYC,ICYCLE,NCON,PARTIT,PRINTX,ITABLE(MXITER)
      REAL*8 LAMPRI
      LOGICAL*1 PARTIT,PRINTX,ITABLE
      INTEGER*4 PRTCYC
 
      COMMON/ANEWDI/CWK,CWKALP,R(MXATOM,MXATOM)
 
      COMMON /STARS/STAR,STAR2,SYMBOL(MXATOM)
      CHARACTER*2 STAR,STAR2,SYMBOL

      COMMON/ENERGY/HSS(MXATOM),HPP(MXATOM),HDD(MXATOM)
 
      LOGICAL*1 FLAG
      INTEGER NEL(9)
 
      IDIST=NATOM-1
      DO 63 I=1,IDIST
        JDIST=I+1
        DO 64 J=JDIST,NATOM
          DX=X(I)-X(J)
          DY=Y(I)-Y(J)
          DZ=Z(I)-Z(J)
          R(I,J)=DSQRT(DX**2+DY**2+DZ**2)
          R(J,I)=R(I,J)
64      CONTINUE
63    CONTINUE
      IF (ONEMAT.AND.IPRINT.LT.-2) RETURN
 
      ICYCLE=1
C
C  SETUP DEFAULT ORBITAL OCCUPATIONS.
C
      IF (L1) GOTO 3
      IH=NELEC/2
      JH=NDIM+1-IH
      DO 1 I=1,JH
1       IOCC(I)=0.0
      DO 2 I=JH,NDIM
2       IOCC(I)=2.0
      IF (IH+IH.NE.NELEC) IOCC(JH-1)=1.0
      GOTO 9
C
C  PROVISION FOR READING IN USER SPECIFIED OCCUPATIONS.
C  ALSO PROVISION FOR NON-INTEGER OCCUPATIONS.
C
 
3     READ(5,2000) (MAXD(I),I=1,NDIM)
2000  FORMAT(78I1)
      TEL=0.0D0
      DO 4 I=1,NDIM
        J=NDIM+1-I
        IOCC(J)=0.0
        IF (MAXD(I).EQ.1) IOCC(J)=1.0
        IF (MAXD(I).EQ.2) IOCC(J)=2.0
        IF (MAXD(I).EQ.3) READ(5,2001) IOCC(J)
2001    FORMAT(F10.8)
4       TEL=TEL+DBLE(IOCC(J))
      TEL=DABS(TEL-DBLE(NELEC))
      IF (TEL.LT.0.001D0) GOTO 9
      WRITE(6,2002)
2002  FORMAT('0*** WARNING ****    ORBITAL POPULATIONS INCONSISTENT WITH
     1 ASSUMED CHARGE ON MOLECULE',////,T10,'I',T25,'IOCC(I)',/)
      DO 99 I=1,NDIM
99      WRITE(6,2003) I,IOCC(I)
2003    FORMAT(T8,I3,T22,F12.8)
      STOP
 
9     CONTINUE
      CON=.5D0*CON
C
C  In case of charge iteration zero some arrays.
C  Effekt des Nullsetzens von SORB auf FMO Berechnung noch unklar. Wird
C  deshalb vorerst nicht ausgefuehrt. 19.11.90/mbs
C
      IF (.NOT.ITERAT) GOTO 15
C       DO 5 I=1,NDIM
C 5       SORB(I)=0.D0
C 19.11.90/mbs
      DO 10 I=1,NATOM
        HSS(I)=0.D0
        HPP(I)=0.D0
        HDD(I)=0.D0
10      COUL0(I)=0.D0
15    IF (.NOT.ONEMAT) GOTO 25
C
C  IN ONE MATRIX CASE, STUFF DIAGONAL ELEMENTS OF S INTO SP.
C
      DO 20 I=1,NDIM
20      SP(I)=S(I,I)
C
C  SETUP DIAGONAL ELEMENTS OF HUCKEL MATRIX IN H(I,J).
C
25    IF (NH.EQ.0) GOTO 35
      ET=COULH
      DO 30 I=1,NH
        IF (ITERAT) ET=COULH-COUL0(I)
30      C(I)=ET
35    IH=NH+1
      ET=0.D0
      DO 50 I=1,NA
        KEYI=KEY(I)
        IF (ITERAT) ET=COUL0(I+NH)
        C(IH)=COULS(KEYI)-ET
        IH=IH+1
        IF (NP(KEYI).EQ.0) GOTO 50
        HH=COULP(KEYI)-ET
        JH=IH+2
        ASSIGN 40 TO IL
        GOTO 42
40      IF (ND(KEYI).EQ.0) GOTO 50
        HH=COULD(KEYI)-ET
        JH=IH+4
        ASSIGN 50 TO IL
42      DO 45 J=IH,JH
45        C(J)=HH
        IH=JH+1
        GOTO IL,(40,50)
50    CONTINUE
      DO 55 I=1,NDIM
55      H(I,I)=C(I)
      IF (NHDG.EQ.1) GOTO 599
      DO 56 I=1,NDIM
56      HDG(I)=C(I)
C
C  SETUP OFF-DIAGONAL ELEMENTS OF HUCKEL MATRIX.
C
C  IPUNCH  .NE. 0  :  CON = 1. + kappa*EXP(-delta*(R-d0))   ( CALL NEWDIA )
C
599   IF (IPUNCH.EQ.0) GOTO 59
C
C Calculate orbital radii.
C
      CALL ORBRAD(NATOM)
      CALL NEWDIA(H,S,NDIM)
      IF (ONEMAT) GOTO 100
      GOTO 806
 
59    CNST=CON
      DO 58 I=2,NDIM
        IL=I-1
        DO 58 J=1,IL
          HH=C(I)+C(J)
          IF (.NOT.L5) GOTO 58
          ET=(C(I)-C(J))/HH
          ET=ET*ET
          CNST=CON+ET/2.0D0+ET*ET*(0.5D0-CON)
58        H(I,J)=CNST*HH*S(I,J)
61    CONTINUE
      IF (ONEMAT) GOTO 100
      DO 60 I=2,NDIM
        IL=I-1
        DO 60 J=1,IL
60        H(J,I)=H(I,J)
C
C  PRINT OUT HUCKEL MATRIX. PRINT OUT TITLE IF METH IS NOT
C  EQUAL TO ZERO.
C
806   IF (ICYCLE.GT.1) GOTO 800
      IF (PRT(6)) GOTO 805
      IF (METH.EQ.0) GOTO 801
      GOTO 802
800   IF (ICYCLE.GE.10000) WRITE(6,701) AB
701   FORMAT ('1RESULTS OF CALCULATION   ',8A8,A6,A2,//)
      IF (PRT(7).OR..NOT.PRINTX) GOTO 805
801   WRITE(6,803)
803   FORMAT('0HUCKEL MATRIX')
      CALL OUTPEG(H,NDIM,NDIM)
      GOTO 805
802   WRITE(6,804)
804   FORMAT('0INPUT HUCKEL MATRIX')
      CALL OUTPEG(H,NDIM,NDIM)
805   IF (ICYCLE.EQ.1.AND.PUN(6)) WRITE(7,825) H
      IF (ICYCLE.GT.1.AND.PUN(7).AND.PRINTX) WRITE(7,825) H
825   FORMAT(8F9.5)
C
C  If calculating energy matrix, store H(JH,JH) in HSS(IH),HPP(IH),
C  HDD(IH). 17.11.90/mbs
C
      IF (ICYCLE.LE.MAXCYC.AND.METH.NE.0) GOTO 100
      IF (NH.EQ.0) GOTO 369
      DO 370 I=1,NH
370     HSS(I)=H(I,I)
369   IH=NH+1
      JH=NH+1
 
      DO 371 I=1,NA
        KEYI=KEY(I)
        HSS(IH)=H(JH,JH)
        JH=JH+1
        IF (NP(KEYI).EQ.0) GOTO 371
        HPP(IH)=H(JH,JH)
        JH=JH+3
        IF (ND(KEYI).EQ.0) GOTO 371
        HDD(IH)=H(JH,JH)
        JH=JH+5
371     IH=IH+1
C
C  If FMO then divide the overlap and the huckel matrix into their
C  fragment overlap and huckel matrices. 29.10.90/mbs
C
100   IF (L6) CALL FMOS(H,S, H1,H2,H3,H4,H5,H6,H7,H8,H9, S1,S2,S3,S4,
     .                  S5,S6,S7,S8,S9, NDIM,NATOM,ND1,ND2,ND3,ND4,ND5,
     .                  ND6,ND7,ND8,ND9)
C
C  Finish the rest of the calculation for the whole molecule.
C  29.10.90/mbs
C
      FLAG=.FALSE.
      CALL HUCK(H,S,MAD,C,SP,COUL0,W,IOCC,HDG, NDIM,NC,NATOM,NTYPE,NHDG)
 
C
C  If FMO then save SORB for the whole molecule. 31.10.90/mbs
C
      IF (L6) THEN
        NATOM1=NATOM+1
        DO 76 I=1,NATOM1
76        SORA(I)=SORB(I)
C
C  Do ICON on fragments. 31.10.90/mbs
C
        IF (NFRAG(1).LE.0) GOTO 900
        CALL FMOR(H1,S1,MAD,C,SP,COUL0,W,IOCC,HDG,
     .            NDIM,NC,NATOM,NTYPE,NHDG,ND1,1)
        NEL(1)=NELEC
 
        IF (NFRAG(2).LE.0) GOTO 900
        CALL FMOR(H2,S2,MAD,C,SP,COUL0,W,IOCC,HDG,
     .            NDIM,NC,NATOM,NTYPE,NHDG,ND2,2)
        NEL(2)=NELEC
 
        IF (NFRAG(3).LE.0) GOTO 900
        CALL FMOR(H3,S3,MAD,C,SP,COUL0,W,IOCC,HDG,
     .            NDIM,NC,NATOM,NTYPE,NHDG,ND3,3)
        NEL(3)=NELEC
 
        IF (NFRAG(4).LE.0) GOTO 900
        CALL FMOR(H4,S4,MAD,C,SP,COUL0,W,IOCC,HDG,
     .            NDIM,NC,NATOM,NTYPE,NHDG,ND4,4)
        NEL(4)=NELEC
 
        IF (NFRAG(5).LE.0) GOTO 900
        CALL FMOR(H5,S5,MAD,C,SP,COUL0,W,IOCC,HDG,
     .            NDIM,NC,NATOM,NTYPE,NHDG,ND5,5)
        NEL(5)=NELEC
 
        IF (NFRAG(6).LE.0) GOTO 900
        CALL FMOR(H6,S6,MAD,C,SP,COUL0,W,IOCC,HDG,
     .            NDIM,NC,NATOM,NTYPE,NHDG,ND6,6)
        NEL(6)=NELEC
 
        IF (NFRAG(7).LE.0) GOTO 900
        CALL FMOR(H7,S7,MAD,C,SP,COUL0,W,IOCC,HDG,
     .            NDIM,NC,NATOM,NTYPE,NHDG,ND7,7)
        NEL(7)=NELEC
 
        IF (NFRAG(8).LE.0) GOTO 900
        CALL FMOR(H8,S8,MAD,C,SP,COUL0,W,IOCC,HDG,
     .            NDIM,NC,NATOM,NTYPE,NHDG,ND8,8)
        NEL(8)=NELEC
 
        IF (NFRAG(9).LE.0) GOTO 900
        CALL FMOR(H9,S9,MAD,C,SP,COUL0,W,IOCC,HDG,
     .            NDIM,NC,NATOM,NTYPE,NHDG,ND9,9)
        NEL(9)=NELEC
 
C
C  If no FMO print options are desired stop here.
C
900     FLAG=.TRUE.
        DO 901 I=1,7
          IF (.NOT.PRIFMO(I)) FLAG=.FALSE.
901     CONTINUE
        IF (FLAG) RETURN
 
C
C  Reload SORB for the whole molecule.
C
        DO 77 I=1,NATOM1
77        SORB(I)=SORA(I)
 
C
C  Do the FMO calculation. S1 works as pointer for the help array U of
C  size (NDIM,NDIM), so S1-S9 will be overwritten. 31.10.90/mbs
C
        CALL FMOSRT(H,S,C,S1,H1,H2,H3,H4,H5,H6,H7,H8,H9, NDIM,NC,NATOM,
     .              ND1,ND2,ND3,ND4,ND5,ND6,ND7,ND8,ND9)
 
C
C  If no additional FMO print options are desired stop here.
C
        FLAG=.TRUE.
        DO 902 I=3,7
          IF (.NOT.PRIFMO(I)) FLAG=.FALSE.
902     CONTINUE
        IF (FLAG) RETURN
 
C
C  Refill the whole molecule occupation. NATOM has new meaning.
C
        NELEC=0
        DO 81 I=1,NATOM
          NELEC=NELEC+NEL(I)
81        NEL(I)=NEL(I)+FMOCH(I)
        IH=NELEC/2
        JH=NDIM-IH
        DO 82 I=1,JH
82        IOCC(I)=0.0
        IF ((IH*2).NE.NELEC) IOCC(I)=1.0
        JH=JH+1
        DO 83 I=JH,NDIM
83        IOCC(I)=2.0
 
C
C  Print out FMO overlap population, charge etc. matrices
C
        FLAG=.TRUE.
        CALL OUTPUT(H,S,MAD,C,COUL0,W,IOCC,HDG, NDIM,NATOM,NTYPE,NC,
     .              NHDG,FLAG,NEL)
        RETURN
      ENDIF
 
101   IF (.NOT.ITERAT) RETURN
220   IF (ICYCLE.GT.MAXCYC) ICYCLE=10000
      IF (METH.GT.1) GOTO 806
      GOTO 15
      END
