C ********************************************************************
C * ICONC - Extended Hueckel Calculations                            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SUBROUTINE MOBY                                                 *
C *                                                                  *
C *  Routine of ICONC(S)                                             *
C *                                                                  *
C *  Usage......: Prints out the datas in MOBY readable format.      *
C *                                                                  *
C *  Author.....: Ruedi Rytz (rr)                                    *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 06. - 11.05.92                                     *
C *  Revision...: last revision 05.05.93/rr/mbs                      *
C *  Remarks....: As the routine OUTPUT overwrites a few datas it    *
C *               became necessary to copy them. Therfore the energy-*
C *               values (E(1:IQ)) are stored in ECOPY(1:IQ). The    *
C *               atomic charges (UB(1:IQ) are stored in             *
C *               UBCOPY(1:IQ). (See routine OUTPUT)                 *
C *                                                                  *
C *  Called from: OUTPUT                                             *
C *                                                                  *
C *  Calls......: -                                                  *
C ********************************************************************

      SUBROUTINE MOBY(ECOPY,H,NDIM,X,Y,Z,NA,SYMBOL,KEY,UBCOPY,R,SORB)

      IMPLICIT NONE
C
C   To make sure that all parameters have been declared  
C
      INTEGER MXATOM,MXITER,MXSETK,MXOMIT,MXAOOC
      INCLUDE 'iconpara.for'
C
C     To get the value of MXATOM
C
          INTEGER     NDIM
          REAL*8      ECOPY(NDIM), H(NDIM,NDIM)
          REAL*8      X(MXATOM), Y(MXATOM), Z(MXATOM)
          INTEGER     NA
          CHARACTER*2 SYMBOL(MXATOM)
          INTEGER     KEY(MXATOM)
          REAL*8      UBCOPY(MXATOM), R(MXATOM,MXATOM)
          INTEGER     SORB(MXATOM)

C
C    NDIM..: Number of MO's                                         
C    NA....: Number of Atoms                                        
C    KEY...: Pointer to the element' s name                         
C    ECOPY.: Contains the energies of the Wavefunctions (copied     
C            values from E)                                         
C    H.....: Huckel-Matrix                                          
C    UBCOPY: Copied charges from UB                                 
C    R.....: Distance-Matrix                                        
C    SYMBOL: Contains the names of the elements                     
C
C CONST
C
      CHARACTER*3 EXTENSION
      PARAMETER  (EXTENSION = '.mo')
      INTEGER     TEN
      PARAMETER  (TEN=10)
C
C VAR
C
      CHARACTER*11 FILNAM
      INTEGER      I, ATNUM, IW, LASTI
      INTEGER      BONDS, J, LOOP
C
C   BONDS.: Number of bonds in molecule  
C
C BEGIN (* Main *)
C
        READ(5, ' (A) ' ) FILNAM
5       IF (FILNAM(1:1) .EQ. ' ') THEN
          FILNAM = FILNAM(2:)
          GOTO 5
        ENDIF

        FILNAM = FILNAM(1:(INDEX(FILNAM,' ') - 1))//EXTENSION

        OPEN (UNIT=3, FILE = FILNAM, ACCESS = 'SEQUENTIAL',
     +           FORM = 'FORMATTED', STATUS = 'UNKNOWN')

        WRITE (3, ' (A2) ') 'MO'
        WRITE (3, ' (A11) ' ) FILNAM
        WRITE (3, ' (I3) ' ) NDIM
        WRITE (3, ' (I3, 2(1X, E13.7E2) ) ')
     +    1, ECOPY(NDIM), ECOPY(1)
C
C            Minimum energy, Maximum energy
C
        WRITE (3, ' (I3, 2(1X, E13.7E2) ) ')
     +    1, 0.0, 1.0

        DO 1 I = 0, (NDIM - 1)
          WRITE (3, ' ( I3, 2(1X, E13.7E2) ) ')
     +      I+1, ECOPY(NDIM - I), 1.0
1       CONTINUE
        WRITE (3, ' (I3) ' ) NA

        J=1
        DO 2 I = 1, NA
          WRITE (3, ' (I3, 3(1X, E13.7E2), 1X, I2, 1X, E13.7E2,
     +                 7X, I1) ' )
     +      ATNUM ( SYMBOL ( KEY(I) ) ), X(I), Y(I), Z(I),
     +      00, UBCOPY(I), 1
2       CONTINUE
C
C  In the next section the connectivity proposal will be calculated.    
C  During the first loop the number of bonds will be stored in the      
C  variable BONDS. While doing the loop for the second time the numbers 
C  of the bonded atoms are written out.                                 
C
        BONDS = 0
        DO 9 LOOP = 1,2
          DO 10 I = 1, NA
            DO 11 J = I+1, NA

              IF      (( ATNUM ( SYMBOL (KEY(I)) ) .LE. 2 ) .AND.
     +                 ( ATNUM ( SYMBOL (KEY(J)) ) .LE. 2 )) THEN
                IF ( R(I,J) .LE. 0.7 ) THEN
                  IF ( LOOP .EQ. 2 ) THEN
                    WRITE (3, ' (2I3) ' ) I, J
                  ELSE
                    BONDS = BONDS + 1
                  ENDIF
                ENDIF

              ELSEIF  (( ATNUM ( SYMBOL (KEY(I)) ) .LE. 2  ) .AND.
     +                 ( ATNUM ( SYMBOL (KEY(J)) ) .LE. 10 )) THEN
                IF ( R(I,J) .LE. 1.2 ) THEN
                  IF ( LOOP .EQ. 2 ) THEN
                    WRITE (3, ' (2I3) ' ) I, J
                  ELSE
                    BONDS = BONDS + 1
                  ENDIF
                ENDIF

              ELSEIF  (( ATNUM ( SYMBOL (KEY(I)) ) .LE. 10 ) .AND.
     +                 ( ATNUM ( SYMBOL (KEY(J)) ) .LE. 2  )) THEN
                IF ( R(I,J) .LE. 1.2 ) THEN
                  IF ( LOOP .EQ. 2 ) THEN
                    WRITE (3, ' (2I3) ' ) I, J
                  ELSE
                    BONDS = BONDS + 1
                  ENDIF
                ENDIF

              ELSEIF  (( ATNUM ( SYMBOL (KEY(I)) ) .LE. 10 ) .AND.
     +                 ( ATNUM ( SYMBOL (KEY(J)) ) .LE. 10 )) THEN
                IF ( R(I,J) .LE. 1.6 ) THEN
                  IF ( LOOP .EQ. 2 ) THEN
                    WRITE (3, ' (2I3) ' ) I, J
                  ELSE
                    BONDS = BONDS + 1
                  ENDIF
                ENDIF

              ELSEIF  (( ATNUM ( SYMBOL (KEY(I)) ) .LE. 2  ) .AND.
     +                 ( ATNUM ( SYMBOL (KEY(J)) ) .LE. 18 )) THEN
                IF ( R(I,J) .LE. 1.6 ) THEN
                  IF ( LOOP .EQ. 2 ) THEN
                    WRITE (3, ' (2I3) ' ) I, J
                  ELSE
                    BONDS = BONDS + 1
                  ENDIF
                ENDIF

              ELSEIF  (( ATNUM ( SYMBOL (KEY(I)) ) .LE. 18 ) .AND.
     +                 ( ATNUM ( SYMBOL (KEY(J)) ) .LE. 2  )) THEN
                IF ( R(I,J) .LE. 1.6 ) THEN
                  IF ( LOOP .EQ. 2 ) THEN
                    WRITE (3, ' (2I3) ' ) I, J
                  ELSE
                    BONDS = BONDS + 1
                  ENDIF
                ENDIF

              ELSEIF  (( ATNUM ( SYMBOL (KEY(I)) ) .LE. 10 ) .AND.
     +                 ( ATNUM ( SYMBOL (KEY(J)) ) .LE. 18 )) THEN
                IF ( R(I,J) .LE. 1.9 ) THEN
                  IF ( LOOP .EQ. 2 ) THEN
                    WRITE (3, ' (2I3) ' ) I, J
                  ELSE
                    BONDS = BONDS + 1
                  ENDIF
                ENDIF

              ELSEIF  (( ATNUM ( SYMBOL (KEY(I)) ) .LE. 18 ) .AND.
     +                 ( ATNUM ( SYMBOL (KEY(J)) ) .LE. 10 )) THEN
                IF ( R(I,J) .LE. 1.9 ) THEN
                  IF ( LOOP .EQ. 2 ) THEN
                    WRITE (3, ' (2I3) ' ) I, J
                  ELSE
                    BONDS = BONDS + 1
                  ENDIF
                ENDIF

              ELSEIF  (( ATNUM ( SYMBOL (KEY(I)) ) .GT. 18 ) .AND.
     +                 ( ATNUM ( SYMBOL (KEY(J)) ) .LE. 2 )) THEN
                IF ( R(I,J) .LE. 1.8 ) THEN
                  IF ( LOOP .EQ. 2 ) THEN
                    WRITE (3, ' (2I3) ' ) I, J
                  ELSE
                    BONDS = BONDS + 1
                  ENDIF
                ENDIF

              ELSEIF  (( ATNUM ( SYMBOL (KEY(I)) ) .LE. 2 ) .AND.
     +                 ( ATNUM ( SYMBOL (KEY(J)) ) .GT. 18)) THEN
                IF ( R(I,J) .LE. 1.8 ) THEN
                  IF ( LOOP .EQ. 2 ) THEN
                    WRITE (3, ' (2I3) ' ) I, J
                  ELSE
                    BONDS = BONDS + 1
                  ENDIF
                ENDIF

              ELSEIF  (( ATNUM ( SYMBOL (KEY(I)) ) .LE. 10 ) .AND.
     +                 ( ATNUM ( SYMBOL (KEY(J)) ) .GT. 18 )) THEN
                IF ( R(I,J) .LE. 2.2 ) THEN
                  IF ( LOOP .EQ. 2 ) THEN
                    WRITE (3, ' (2I3) ' ) I, J
                  ELSE
                    BONDS = BONDS + 1
                  ENDIF
                ENDIF

              ELSEIF  (( ATNUM ( SYMBOL (KEY(I)) ) .GT. 18 ) .AND.
     +                 ( ATNUM ( SYMBOL (KEY(J)) ) .LE. 10 )) THEN
                IF ( R(I,J) .LE. 2.2 ) THEN
                  IF ( LOOP .EQ. 2 ) THEN
                    WRITE (3, ' (2I3) ' ) I, J
                  ELSE
                    BONDS = BONDS + 1
                  ENDIF
                ENDIF

              ELSEIF ( R(I,J) .LE. 2.6 ) THEN
                IF ( LOOP .EQ. 2 ) THEN
                  WRITE (3, ' (2I3) ' ) I, J
                ELSE
                  BONDS = BONDS + 1
                ENDIF
              ENDIF

11          CONTINUE
10        CONTINUE
          IF ( LOOP .EQ. 1 ) THEN
            WRITE (3, ' (I3) ' ) BONDS
          ENDIF
9       CONTINUE

        WRITE (3, ' (A) ' ) 'EHMO-Rechnung'
        WRITE (3, ' (2I3) '  ) NA, NDIM

        IW = 1
        DO 30 I = 1, NDIM
          IF (SORB(IW) .EQ. I) THEN
            WRITE (3, ' (2I3) ') IW, IW
            LASTI = I
          ENDIF
          IF (SORB(IW + 1) .EQ. I + 1) THEN
            WRITE (3, ' (2I3) ') LASTI, I
            IW = IW + 1
          ENDIF
30      CONTINUE
C
C  IW contains the number of the atom. LASTI is the number of the first 
C  AO centered on the (IW+1)th atom.                                    
C
        DO 40 I = 0, (NDIM - 1)
          WRITE (3, ' (2X, F8.4, I3) ') ECOPY (NDIM - I), I + 1
          WRITE (3, ' ( 10(1X, F7.4)) ') (H(J,(NDIM-I)), J=1,NDIM)
          IF ( MOD(NDIM,TEN) .LT. 1.0E-5 ) THEN
            WRITE (3, ' (A) ')
          ENDIF
40      CONTINUE

        CLOSE (3, STATUS = 'KEEP')
      END
C
C END (*Main*)
C
      INTEGER   FUNCTION ATNUM ( SYM )
C
C      Returns the atomic number of a given element    
C      More elements can be added easily when needed   
C
        IMPLICIT NONE 

        CHARACTER*2 SYM

        IF ( SYM .EQ. ' H') THEN
          ATNUM = 1
        ELSEIF (SYM .EQ. 'He') THEN
          ATNUM = 2
        ELSEIF (SYM .EQ. 'Li') THEN
          ATNUM = 3
        ELSEIF (SYM .EQ. 'Be') THEN
          ATNUM = 4
        ELSEIF (SYM .EQ. ' B') THEN
          ATNUM = 5
        ELSEIF (SYM .EQ. ' C') THEN
          ATNUM = 6
        ELSEIF (SYM .EQ. ' N') THEN
          ATNUM = 7
        ELSEIF (SYM .EQ. ' O') THEN
          ATNUM = 8
        ELSEIF (SYM .EQ. ' F') THEN
          ATNUM = 9
        ELSEIF (SYM .EQ. 'Ne') THEN
          ATNUM = 10
        ELSEIF (SYM .EQ. 'Na') THEN
          ATNUM = 11
        ELSEIF (SYM .EQ. 'Mg') THEN
          ATNUM = 12
        ELSEIF (SYM .EQ. 'Al') THEN
          ATNUM = 13
        ELSEIF (SYM .EQ. 'Si') THEN
          ATNUM = 14
        ELSEIF (SYM .EQ. ' P') THEN
          ATNUM = 15
        ELSEIF (SYM .EQ. ' S') THEN
          ATNUM = 16
        ELSEIF (SYM .EQ. 'Cl') THEN
          ATNUM = 17
        ELSEIF (SYM .EQ. 'Ar') THEN
          ATNUM = 18
        ELSE
          ATNUM = 19
        ENDIF
      END
