C ********************************************************************
C * ICONC - Extended Hueckel Calculations                            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************

C ********************************************************************
C *                                                                  *
C *  SURBROUTINE RECALL                                              *
C *                                                                  *
C *  Routine of ICONC(S)                                             *
C *                                                                  *
C *  Usage......: Calculates integrals for core-core repulsion:      *
C *               1/R-exp(-2*zeta*R)/(2*n*R) *                       *
C *               * sum over [(2*R*zeta)**(2*n-i)*i/(2*n-i)]         *
C *                                                                  *
C *  Authors....: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Department of Chemistry and Biochemistry           *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: -                                                  *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: REP                                                *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE RECALL(N,R,SE,SUM)
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER ML1
 
      ML=2*N
      SUM=0
      IF (SE.EQ.0) GOTO 99
C
C  In the case of single zeta d-orbitals (R*SE)**ML1 equals 0**0,
C  so jump to end of routine
C
      DO 10 I=1,ML
        ML1=ML-I
        FAK=1
        DO 20 I2=1,ML1
          FAK=FAK*I2
20      CONTINUE
        RES=I/FAK
C
C  SE is slaterexponent * 2
C  In case of the cross term of double-zeta orbitals SE is exp1+exp2
C
        RES1=(R*SE)**ML1
        SUM=SUM+RES*RES1
10    CONTINUE
      RES2=(DEXP(-SE*R))/(2*N*R)
      SUM=RES2*SUM
99    CONTINUE
      RETURN
      END
